import { Component, Input, Output, EventEmitter } from '@angular/core';
import { TransitionController } from '../transition/transition-controller';
import { Transition } from '../transition/transition';
var SuiAccordionPanel = (function () {
    function SuiAccordionPanel() {
        this.transitionController = new TransitionController(false);
        this._isOpen = false;
        this.isOpenChange = new EventEmitter(false);
    }
    Object.defineProperty(SuiAccordionPanel.prototype, "service", {
        /**
         * @param {?} service
         * @return {?}
         */
        set: function (service) {
            this._service = service;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiAccordionPanel.prototype, "isOpen", {
        /**
         * @return {?}
         */
        get: function () {
            return this._isOpen;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            // Convert to boolean (fixes false != undefined)
            value = !!value;
            if (value != this.isOpen) {
                // Only update if the value has changed.
                this._isOpen = value;
                if (value && this._service) {
                    // If we are opening this panel, we must close the other ones.
                    this._service.closeOtherPanels(this);
                }
                this.isOpenChange.emit(this.isOpen);
                // Cancel all current animations, and fade the contents. The direction is automatic.
                this.transitionController.stopAll();
                this.transitionController.animate(new Transition(this.transition, this.transitionDuration));
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiAccordionPanel.prototype, "transition", {
        /**
         * @return {?}
         */
        get: function () {
            if (this._service) {
                return this._service.transition;
            }
            return "fade";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiAccordionPanel.prototype, "transitionDuration", {
        /**
         * @return {?}
         */
        get: function () {
            if (this._service) {
                // Return the service defined transition duration.
                return this._service.transitionDuration;
            }
            // Revert to instantaneous if the service is not yet loaded.
            return 0;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SuiAccordionPanel.prototype.toggle = function () {
        if (!this.isDisabled) {
            this.isOpen = !this.isOpen;
        }
    };
    return SuiAccordionPanel;
}());
export { SuiAccordionPanel };
SuiAccordionPanel.decorators = [
    { type: Component, args: [{
                selector: 'sui-accordion-panel',
                exportAs: 'suiAccordionPanel',
                template: "\n<!-- Title -->\n<div class=\"title\" [class.active]=\"isOpen\" (click)=\"toggle()\" >\n    <ng-content select=\"[title]\"></ng-content>\n</div>\n<!-- Content -->\n<div [suiCollapse]=\"!isOpen\" [collapseDuration]=\"transitionDuration\">\n    <div class=\"content\" [class.active]=\"isOpen\" [suiTransition]=\"transitionController\">\n        <ng-content select=\"[content]\"></ng-content>\n    </div>\n</div>\n",
                styles: ["\n/* Manual style as Semantic UI relies on > selector */\n.content {\n    padding: .5em 0 1em;\n}\n\n/* Another > selector fix */\n:host:first-child .title {\n    border-top: none;\n}\n"]
            },] },
];
/**
 * @nocollapse
 */
SuiAccordionPanel.ctorParameters = function () { return []; };
SuiAccordionPanel.propDecorators = {
    'isDisabled': [{ type: Input },],
    'isOpen': [{ type: Input },],
    'isOpenChange': [{ type: Output },],
};
function SuiAccordionPanel_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiAccordionPanel.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiAccordionPanel.ctorParameters;
    /** @type {?} */
    SuiAccordionPanel.propDecorators;
    /** @type {?} */
    SuiAccordionPanel.prototype._service;
    /** @type {?} */
    SuiAccordionPanel.prototype.transitionController;
    /** @type {?} */
    SuiAccordionPanel.prototype.isDisabled;
    /** @type {?} */
    SuiAccordionPanel.prototype._isOpen;
    /** @type {?} */
    SuiAccordionPanel.prototype.isOpenChange;
}
//# sourceMappingURL=accordion-panel.js.map