import { Injectable, ApplicationRef, ComponentFactoryResolver, Injector, ReflectiveInjector } from '@angular/core';
import { TemplateModalConfig, ComponentModalConfig } from './modal-config';
import { SuiModal } from './modal';
import { Modal } from './modal-controls';
import { ActiveModal } from './active-modal';
var SuiModalService = (function () {
    /**
     * @param {?} _applicationRef
     * @param {?} _componentFactoryResolver
     * @param {?} _injector
     */
    function SuiModalService(_applicationRef, _componentFactoryResolver, _injector) {
        this._applicationRef = _applicationRef;
        this._componentFactoryResolver = _componentFactoryResolver;
        this._injector = _injector;
    }
    /**
     * @template T, U, V
     * @param {?} modal
     * @return {?}
     */
    SuiModalService.prototype.open = function (modal) {
        // Resolve factory for creating `SuiModal` components.
        var /** @type {?} */ factory = this._componentFactoryResolver.resolveComponentFactory(SuiModal);
        // Generate a component using the injector and the previously resolved factory.
        var /** @type {?} */ componentRef = factory.create(this._injector);
        // Shorthand for the created modal component instance.
        var /** @type {?} */ modalComponent = componentRef.instance;
        // If the config is for a template based modal,
        if (modal instanceof TemplateModalConfig) {
            // Inject the template into the view.
            componentRef.instance.templateSibling.createEmbeddedView(modal.template, {
                // `let-context`
                $implicit: modal.context,
                // `let-modal="modal"`
                modal: componentRef.instance.controls
            });
        }
        else if (modal instanceof ComponentModalConfig) {
            // Resolve factory for creating a new instance of the provided component.
            var /** @type {?} */ contentComponentFactory = this._componentFactoryResolver.resolveComponentFactory(/** @type {?} */ (modal.component));
            // Provide an instance of `Modal` for the injector, to be used in the component constructor.
            var /** @type {?} */ modalContentInjector = ReflectiveInjector.resolveAndCreate([
                {
                    provide: Modal,
                    useValue: new Modal(modalComponent.controls, modal.context)
                }
            ], this._injector);
            // Generate a component using the custom injector and the previously resolved factory.
            var /** @type {?} */ contentComponentRef = contentComponentFactory.create(modalContentInjector);
            // Insert the new component into the content of the modal.
            modalComponent.templateSibling.insert(contentComponentRef.hostView);
            // Shorthand for access to the content component's DOM element.
            var /** @type {?} */ contentElement = (contentComponentRef.location.nativeElement);
            // Move all of the DOM elements inside the component to the main modal element.
            // This is done so that CSS classes apply correctly. It does stop any custom styles from working however, so other ways may have to be investigated.
            while (contentElement.hasChildNodes()) {
                contentElement.parentElement.appendChild(contentElement.removeChild(contentElement.firstChild));
            }
            // Remove the generated component's 'empty shell' from the DOM.
            contentElement.remove();
        }
        // Attach the new modal component to the application.
        this._applicationRef.attachView(componentRef.hostView);
        // Move the new modal component DOM to the document body.
        document.querySelector("body").appendChild(componentRef.location.nativeElement);
        // Initialise the generated modal with the provided config.
        modalComponent.loadConfig(modal);
        // Return an instance of an `ActiveModal`, so the user can control the new modal.
        return new ActiveModal(modal, componentRef);
    };
    return SuiModalService;
}());
export { SuiModalService };
SuiModalService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
SuiModalService.ctorParameters = function () { return [
    { type: ApplicationRef, },
    { type: ComponentFactoryResolver, },
    { type: Injector, },
]; };
function SuiModalService_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiModalService.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiModalService.ctorParameters;
    /** @type {?} */
    SuiModalService.prototype._applicationRef;
    /** @type {?} */
    SuiModalService.prototype._componentFactoryResolver;
    /** @type {?} */
    SuiModalService.prototype._injector;
}
//# sourceMappingURL=modal.service.js.map