import { Directive, Input, ElementRef, ComponentFactoryResolver, ViewContainerRef, HostListener } from '@angular/core';
import { SuiPopup } from './popup';
import { parseBooleanAttribute } from '../util/util';
import { PopupConfig, PopupTrigger } from './popup-config';
import { SuiPopupConfig } from './popup.service';
var SuiPopupDirective = (function () {
    /**
     * @param {?} _element
     * @param {?} _viewContainerRef
     * @param {?} _componentFactoryResolver
     * @param {?} popupDefaults
     */
    function SuiPopupDirective(_element, _viewContainerRef, _componentFactoryResolver, popupDefaults) {
        this._element = _element;
        this._viewContainerRef = _viewContainerRef;
        this._componentFactoryResolver = _componentFactoryResolver;
        this.config = new PopupConfig(popupDefaults);
    }
    Object.defineProperty(SuiPopupDirective.prototype, "popupTemplate", {
        /**
         * @param {?} template
         * @return {?}
         */
        set: function (template) {
            this.config.template = template;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupHeader", {
        /**
         * @param {?} header
         * @return {?}
         */
        set: function (header) {
            this.config.header = header;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupText", {
        /**
         * @param {?} text
         * @return {?}
         */
        set: function (text) {
            this.config.text = text;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupInverted", {
        /**
         * @param {?} inverted
         * @return {?}
         */
        set: function (inverted) {
            this.config.isInverted = parseBooleanAttribute(inverted);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupBasic", {
        /**
         * @param {?} basic
         * @return {?}
         */
        set: function (basic) {
            this.config.isBasic = parseBooleanAttribute(basic);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupTransition", {
        /**
         * @param {?} transition
         * @return {?}
         */
        set: function (transition) {
            this.config.transition = transition;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupTransitionDuration", {
        /**
         * @param {?} duration
         * @return {?}
         */
        set: function (duration) {
            this.config.transitionDuration = duration;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupPlacement", {
        /**
         * @param {?} placement
         * @return {?}
         */
        set: function (placement) {
            this.config.placement = placement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupDelay", {
        /**
         * @param {?} delay
         * @return {?}
         */
        set: function (delay) {
            this.config.delay = delay;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupTrigger", {
        /**
         * @return {?}
         */
        get: function () {
            return this.config.trigger;
        },
        /**
         * @param {?} trigger
         * @return {?}
         */
        set: function (trigger) {
            this.config.trigger = trigger;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupConfig", {
        /**
         * @param {?} config
         * @return {?}
         */
        set: function (config) {
            this.config.batch(config);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "_popup", {
        /**
         * @return {?}
         */
        get: function () {
            return this._componentRef.instance;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.open = function () {
        var _this = this;
        // Cancel the opening timer.
        clearTimeout(this._openingTimeout);
        // Start the popup opening after the specified delay interval.
        this._openingTimeout = window.setTimeout(function () {
            if (!_this._componentRef) {
                // Resolve component factory for the `SuiPopup` component.
                var /** @type {?} */ factory = _this._componentFactoryResolver.resolveComponentFactory(SuiPopup);
                // Generate a component using the view container reference and the previously resolved factory.
                _this._componentRef = _this._viewContainerRef.createComponent(factory);
                // If there is a template, inject it into the view.
                if (_this.config.template) {
                    _this._popup.templateSibling.createEmbeddedView(_this.config.template, { $implicit: _this._popup });
                }
                // Configure popup with provided config, and attach a reference to the anchor element.
                _this._popup.config = _this.config;
                _this._popup.anchor = _this._element;
                // Move the generated element to the body to avoid any positioning issues.
                document.querySelector("body").appendChild(_this._componentRef.location.nativeElement);
                // When the popup is closed (onClose fires on animation complete),
                _this._popup.onClose.subscribe(function () {
                    // Destroy the component reference (which removes the popup from the DOM).
                    _this._componentRef.destroy();
                    // Unset the reference pointer to enable a new popup to be created on next open.
                    _this._componentRef = null;
                });
            }
            // Start popup open transition.
            _this._popup.open();
        }, this.config.delay);
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.close = function () {
        // Cancel the opening timer to stop the popup opening after close has been called.
        clearTimeout(this._openingTimeout);
        if (this._componentRef) {
            // Start popup close transition.
            this._popup.close();
        }
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.toggle = function () {
        // If the popup hasn't been created, or it has but it isn't currently open, open the popup.
        if (!this._componentRef || (this._componentRef && !this._popup.isOpen)) {
            return this.open();
        }
        // O'wise, close it.
        return this.close();
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.onMouseEnter = function () {
        if (this.popupTrigger == PopupTrigger.Hover) {
            this.open();
        }
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.onMouseLeave = function () {
        if (this.popupTrigger == PopupTrigger.Hover) {
            this.close();
        }
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.onClick = function () {
        if (this.popupTrigger == PopupTrigger.Click || this.popupTrigger == PopupTrigger.OutsideClick) {
            // Repeated clicks require a toggle, rather than just opening the popup each time.
            this.toggle();
        }
    };
    /**
     * @param {?} e
     * @return {?}
     */
    SuiPopupDirective.prototype.onDocumentClick = function (e) {
        // If the popup trigger is outside click,
        if (this._componentRef && this.popupTrigger == PopupTrigger.OutsideClick) {
            var /** @type {?} */ target = (e.target);
            // Close the popup if the click is outside of the popup element.
            if (!((this._element.nativeElement)).contains(target)) {
                this.close();
            }
        }
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.onFocus = function () {
        if (this.popupTrigger == PopupTrigger.Focus) {
            this.open();
        }
    };
    /**
     * @return {?}
     */
    SuiPopupDirective.prototype.onFocusOut = function () {
        if (this.popupTrigger == PopupTrigger.Focus) {
            this.close();
        }
    };
    return SuiPopupDirective;
}());
export { SuiPopupDirective };
SuiPopupDirective.decorators = [
    { type: Directive, args: [{
                selector: '[suiPopup]',
                exportAs: 'suiPopup'
            },] },
];
/**
 * @nocollapse
 */
SuiPopupDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: ComponentFactoryResolver, },
    { type: SuiPopupConfig, },
]; };
SuiPopupDirective.propDecorators = {
    'popupTemplate': [{ type: Input },],
    'popupHeader': [{ type: Input },],
    'popupText': [{ type: Input },],
    'popupInverted': [{ type: Input },],
    'popupBasic': [{ type: Input },],
    'popupTransition': [{ type: Input },],
    'popupTransitionDuration': [{ type: Input },],
    'popupPlacement': [{ type: Input },],
    'popupDelay': [{ type: Input },],
    'popupTrigger': [{ type: Input },],
    'popupConfig': [{ type: Input },],
    'onMouseEnter': [{ type: HostListener, args: ["mouseenter",] },],
    'onMouseLeave': [{ type: HostListener, args: ["mouseleave",] },],
    'onClick': [{ type: HostListener, args: ["click",] },],
    'onDocumentClick': [{ type: HostListener, args: ["document:click", ["$event"],] },],
    'onFocus': [{ type: HostListener, args: ["focus",] },],
    'onFocusOut': [{ type: HostListener, args: ["focusout",] },],
};
function SuiPopupDirective_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiPopupDirective.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiPopupDirective.ctorParameters;
    /** @type {?} */
    SuiPopupDirective.propDecorators;
    /** @type {?} */
    SuiPopupDirective.prototype.config;
    /** @type {?} */
    SuiPopupDirective.prototype._componentRef;
    /** @type {?} */
    SuiPopupDirective.prototype._openingTimeout;
    /** @type {?} */
    SuiPopupDirective.prototype._element;
    /** @type {?} */
    SuiPopupDirective.prototype._viewContainerRef;
    /** @type {?} */
    SuiPopupDirective.prototype._componentFactoryResolver;
}
//# sourceMappingURL=popup.directive.js.map